<?php

/**
 * This class represents category
 */
class CCategoryTree{
	
	//-------------------------------------------------------------------------
	/**
	 * Constructor
	 */
	public function __construct($database, $restrictions = null){
		
		$this->db = $database;
		$this->expandRestrictions($restrictions);
		$this->init();
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Return all categories as tree
	 * @return array with categories as tree
	 */
	public function getRootNodes(){
		
		return $this->tree;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Return given node
	 * @return array with categories as tree
	 */
	public function getNode($id){
		
		$node = $this->findNode($this->tree, $id);
		return $node;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Return given node
	 * @return array with categories as tree
	 */
	private function findNode(&$tree, $node_id){
		
		foreach($tree as $node){
			
			if($node['id'] == $node_id)
				return $node;
			
			$child = $this->findNode($node['nodes'], $node_id);
			if($child != null)
				return $child;
		}
		
		return null;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Init tree
	 * @return array with categories as tree
	 */
	private function init(){

		// create tempoary table			
 		$sql = "CREATE TEMPORARY TABLE rc " . $this->buildQuery(); 

		$result = $this->db->query($sql);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") :" . $result->getMessage());


 		$sql = "SELECT categories.cat_id, cat_name, parent_id, count( rc.res_id ) " .
        	 "FROM categories " . 
        	 "LEFT JOIN rc ON categories.cat_id = rc.cat_id " .
        	 "GROUP BY categories.cat_id ORDER BY categories.cat_name";


		$result = $this->db->query($sql);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") :" . $result->getMessage());
    	
    $categories = array();
		while($row = $result->fetchRow()){
			$category['id'] = $row[0];
			$category['name'] = $row[1];
			$category['parent'] = $row[2];
			$category['count'] = $row[3];
			
			if(!isset($categories[$category['parent']]))
				$categories[$category['parent']] = array();
				
			array_push($categories[$category['parent']], $category);
		}
		
		$parents = array();
		$this->tree = $this->convertToTree($categories, 0, $parents);
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Convert array with categories to tree
	 *
	 * @param $categories array with categories
	 * @param $parent_id parent id
	 * @return array as tree
	 */
	private function convertToTree(&$categories, $parent_id, $parents){
	
		$tree = array();
		if(isset($categories[$parent_id])){
			for($i = 0; $i < count($categories[$parent_id]) ; $i++){
			
				$category = $categories[$parent_id][$i];
	
				$item['id'] = $category['id'];
				$item['name'] = $category['name'];
				$item['parents'] = $parents;
				$new_parents = $parents;
				array_push($new_parents, $item['id']);
				$item['nodes'] = $this->convertToTree($categories, $category['id'], $new_parents);
				$count = 0;
				foreach($item['nodes'] as $node){
					$count += $node['count'];
				}				
				$item['count'] = $category['count'] + $count;
				$tree[$item['id']] = $item;
			}
		}		
		return $tree;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Split restrictions to keywords and category
	 * Get all category id
	 */
	private function expandRestrictions($restrictions){

		$this->keywords = array();
		$this->categories = array();
		
		if(is_array($restrictions)){
			foreach($restrictions as $restriction){
				if($restriction['type'] == "keyword"){
					array_push($this->keywords, trim($restriction['data']));
				}else{
					array_push($this->categories, $restriction['data']);
				}
			}
		}		
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * build query
	 */
	private function buildQuery(){

		$keyword_count = count($this->keywords);
		$category_count = count($this->categories);
		
		$query = "SELECT resources_categories.cat_id, resources_categories.res_id FROM resources_categories ";
		
		if($keyword_count + $category_count == 0)
			return $query;
		
		// Add keyword tables
		for($i = 0; $i < $keyword_count; $i++){
			$query .= ", resources_keywords AS rk$i, keywords AS k$i ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			$query .= ", resources_categories AS rc$i ";
		}
		
		$query .= "WHERE ";
		$add_and = false;

		for($i = 0; $i < $keyword_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources_categories.res_id=rk$i.res_id AND rk$i.key_id=k$i.key_id ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources_categories.res_id=rc$i.res_id ";
		}

		$i = 0;
		foreach($this->keywords as $keyword){
			$query .= " AND k$i.key_name='$keyword' ";
			$i ++;
		}
		
		$i = 0;
		foreach($this->categories as $category){
			$item = new CCategory($this->db, $category);
			$ids = $item->getAllChildren();

			$query .= " AND (rc$i.cat_id=$category ";
			foreach($ids as $id){
				$query .= " OR rc$i.cat_id=$id ";
			}
			$query .= " ) ";

			$i ++;
		}
		
		return $query;
	}	
	
	
	//---------------------------------------------------------------------------
	// Private members
	
	/** Database */
	private $db;
	/** Restrictions to the result set */
	private	$keywords;
	private	$categories;
	/** Tree data */
	private $tree;
}	
?>